API_KEY_ENV: str = "CONFIDENT_AI_API_KEY"
LOG_TO_SERVER_ENV: str = "DO_NOT_SEND_TO_CONFIDENT_AI"
IMPLEMENTATION_ID_ENV: str = "CONFIDENT_AI_IMP_ID"
IMPLEMENTATION_ID_NAME: str = "CONFIDENT_AI_IMP_NAME"
